<?php

namespace Drupal\billwerk_subscriptions_subscriptionjs\Form;

use Drupal\billwerk_subscriptions\Environment;
use Drupal\billwerk_subscriptions\SettingsHelper;
use Drupal\billwerk_subscriptions\Subscriber;
use Drupal\billwerk_subscriptions_entities\SubscriptionPlansManager;
use Drupal\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * The premium downgrade form.
 */
class SubscriptionChangeForm extends FormBase {

  public function __construct(
    // @improve:
		// These should be "readonly", but that's currently not possible on
		// multipage forms due to a dependency injection bug leading to an error:
		// Error: Cannot initialize readonly property Drupal\billwerk_subscriptions_subscriptionjs\Form\BillwerkSubscriptionjsFormAbstract::$account from scope Drupal\Core\Form\FormBase in Drupal\Core\Form\FormBase->__wakeup() (line 89 of core/lib/Drupal/Core/DependencyInjection/DependencySerializationTrait.php).
		protected AccountInterface $account,
		protected SubscriptionPlansManager $subscriptionPlansManager,
		protected Environment $environment,
    ) {

	}

	/**
	 * {@inheritdoc}
	 */
	public static function create(ContainerInterface $container) {
		return new static(
			$container->get('current_user'),
			$container->get('billwerk_subscriptions_entities.subscription_plans_manager'),
			$container->get('billwerk_subscriptions.environment'),
		);
	}

	/**
	 * {@inheritdoc}
	 */
	public function getFormId() {
		return 'billwerk_subscriptions_subscriptionjs_subscription_form';
	}

	/**
	 * {@inheritdoc}
	 */
	public function buildForm(array $form, FormStateInterface $form_state) {
		switch ($form_state->get('step')) {
			case 'review':
				$form = $this->buildReviewForm($form, $form_state);
				break;

			case 'buy':
				$form = $this->buildBuyForm($form, $form_state);
				break;

			default:
				$form = $this->buildSelectionForm($form, $form_state);
				break;
		}

    // Add this library to prevent duplicate form submits:
		$form['#attached']['library'][] = 'core/drupal.form';

		return $form;
	}

	protected function buildSelectionForm(array $form, FormStateInterface $form_state) {
		$subscriber = Subscriber::loadByDrupalUid($this->currentUser()->id());
		$subscriberCurrentPlanVariantId = $subscriber->getBillwerkContract()->getBillwerkContractSubscription()->getPlanVariantId();
		$subscriberCurrentPlanVariantEntity = $this->subscriptionPlansManager->getBillwerkPlanVariantByPlanVariantId($subscriberCurrentPlanVariantId, $this->environment);
		$subscriberCurrentComponentSubscriptions = $subscriber->getBillwerkContract()->getBillwerkContractSubscription()->getComponentSubscriptions();

		$form['planvariant_current'] = [
			'#type' => 'item',
			'#title' => $this->t('Your current subscription plan:'),
			'#markup' => $subscriberCurrentPlanVariantEntity->labelTranslated(),
		];

		$options = $this->subscriptionPlansManager->getPlanVariantsAllSelectOptions();
		if (!empty($options)) {
			// We're using the machine names as keys to not expose the Billwerk IDs:
			$form['planvariant'] = [
				'#type' => 'radios',
				'#title' => $this->t('Subscription'),
				'#options' => $options,
				'#default_value' => $subscriberCurrentPlanVariantEntity->getMachineName(),
				'#description' => $this->t('Select the subscription plan for the upgrade'),
				'#required' => TRUE,
				'#ajax' => [
					'callback' => '::updatePlanDetailsCallback',
					'wrapper' => 'edit-plan-details',
					'progress' => [
						'type' => 'throbber',
						'message' => $this->t('Loading plan details ...'),
					],
				],
			];

			$form['planvariant_details'] = [
				'#theme' => 'plan_variant_details',
				'#prefix' => '<div id="edit-plan-details">',
				'#suffix' => '</div>',
			];
			if ($form_state->hasValue('planvariant')) {
				$planVariantKey = $form_state->getValue('planvariant');
				$form['planvariant_details']['#label'] = $this->subscriptionPlansManager->getBillwerkPlanVariantByMachineName($planVariantKey)->labelTranslated();
			}

			// Components:
			$currentAddons = [];
			if (!empty($subscriberCurrentComponentSubscriptions)) {
				foreach ($subscriberCurrentComponentSubscriptions as $subscriberCurrentComponentSubscription) {
					$subscriberCurrentPlanVariantEntity = $this->subscriptionPlansManager->getBillwerkComponentByComponentId($subscriberCurrentComponentSubscription->getComponentId(), $this->environment);
					$currentAddons[] = $subscriberCurrentPlanVariantEntity->getMachineName();
				}
			}
			// We're using the machine names as keys to not expose the Billwerk IDs:
			$form['components'] = [
				'#type' => 'checkboxes',
				'#title' => $this->t('Addons'),
				'#options' => $this->subscriptionPlansManager->getComponentsAllSelectOptions(),
				'#default_value' => $currentAddons,
				'#multiple' => TRUE,
				'#description' => $this->t('Select the add-ons to add to your subscription.'),
				'#required' => TRUE,
			];

			$form['couponcode'] = [
				'#type' => 'textfield',
				'#title' => $this->t('Coupon code'),
				'#description' => $this->t('Do you have a coupon code?'),
				'#default_value' => '',
				'#required' => FALSE,
			];

			$form['actions']['#type'] = 'actions';
			$form['actions']['proceed'] = [
				'#type' => 'submit',
				'#value' => $this->t('Proceed'),
				'#submit' => ['::submitSelectionForm'],
				'#button_type' => 'primary',
			];
		}
		else {
			$form['message'] = [
				'#type' => 'item',
				'#markup' => $this->t('No change possible for your subscription. Please contact us.'),
			];
		}

		return $form;
	}

	protected function buildReviewForm(array $form, FormStateInterface $form_state): array {
    $subscriber = Subscriber::loadByDrupalUid($this->currentUser()->id());

    // @todo: Move to buy:
    $form['subscriptionjs_wrapper'] = [
			'#type' => 'item',
			'#markup' => '<div id="subscriptionjs-wrapper"></div>',
		];

		$form['actions']['#type'] = 'actions';
		$form['actions']['submit'] = [
			'#type' => 'submit',
      '#submit' => ['::submitReviewForm'],
			'#value' => $this->t('Proceed'),
			'#button_type' => 'primary',
		];

		$form['#attached']['library'][] = 'billwerk_subscriptions_subscriptionjs/subscriptionjs';
		$form['#attached']['library'][] = 'billwerk_subscriptions_subscriptionjs/subscriptionjs.init';
		$form['#attached']['drupalSettings']['billwerk']['selfserviceToken'] = $subscriber->billwerkGetSelfserviceToken();
    $form['#attached']['drupalSettings']['billwerk']['selfservicePublicApiKey'] = $this->environment->getSelfservicePublicApiKey();


		// @todo:
		$selectedPlanVariantMachineName = $form_state->getValue('planvariant');
		$newPlanVariantEntitiy = $this->subscriptionPlansManager->getBillwerkPlanVariantByMachineName($selectedPlanVariantMachineName);
		$selectedComponentsMachineNames = $form_state->getValue('components');
		if (!empty($selectedComponentsMachineNames)) {
			foreach ($selectedComponentsMachineNames as $selectedComponentsMachineName) {
				// $newComponentsEntities = $this->subscriptionPlansManager->getBillwerkComponentByComponentId($subscriberCurrentComponentSubscription->getComponentId(), $this->environment);
			}
		}
		$selectedCouponCode = $form_state->getValue('couponcode');

		// @todo: Finish:
		$billwerkOrderObj = $subscriber->billwerkChangeSubscription($newPlanVariantEntitiy->getBillwerkId($this->environment->getEnvironmentName()), [], [], $selectedCouponCode);
    dsm($billwerkOrderObj);
    $form['#attached']['drupalSettings']['billwerk']['orderId'] = $billwerkOrderObj->Id;

    $form['preview'] = self::billwerkOrderDtoToRenderArray($billwerkOrderObj);

		return $form;
	}

  protected static function billwerkOrderDtoToRenderArray(\stdClass $billwerkOrderObj) {
    return [
      '#theme' => 'billwerk_order',
      '#id' => $billwerkOrderObj->Id,
      '#planVariantId' => $billwerkOrderObj->PlanVariantId ?? NULL,
      '#planId' => $billwerkOrderObj->PlanId ?? NULL,
      '#allowWithoutPaymentData' => $billwerkOrderObj->AllowWithoutPaymentData ?? NULL,
      '#lineitems' => [
        '#theme' => 'billwerk_order_lineitems',
        '#description' => $billwerkOrderObj->Lineitems->Description ?? NULL,
        '#quantity' => $billwerkOrderObj->Lineitems->Quantity ?? NULL,
        '#pricePerUnit' => $billwerkOrderObj->Lineitems->PricePerUnit ?? NULL,
        '#vatPercentage' => $billwerkOrderObj->Lineitems->VatPercentage ?? NULL,
        '#periodStart' => $billwerkOrderObj->Lineitems->PeriodStart ?? NULL,
        '#periodEnd' => $billwerkOrderObj->Lineitems->PeriodEnd ?? NULL,
        '#periodMultiplier' => $billwerkOrderObj->Lineitems->PeriodMultiplier ?? NULL,
        '#scaleAmount' => $billwerkOrderObj->Lineitems->ScaleAmount ?? NULL,
        '#productId' => $billwerkOrderObj->Lineitems->ProductId ?? NULL,
        '#totalNet' => $billwerkOrderObj->Lineitems->TotalNet ?? NULL,
        '#totalVat' => $billwerkOrderObj->Lineitems->TotalVat ?? NULL,
        '#totalGross' => $billwerkOrderObj->Lineitems->TotalGross ?? NULL,
        '#productDescription' => $billwerkOrderObj->Lineitems->ProductDescription ?? NULL,
        '#attributes' => [],
      ],
      '#coupon' => [],
      '#currency' => $billwerkOrderObj->Currency ?? NULL,
      '#total' => $billwerkOrderObj->Total ?? NULL,
      '#totalVat' => $billwerkOrderObj->TotalVat ?? NULL,
      '#totalGross' => $billwerkOrderObj->TotalGross ?? NULL,
      '#isTrial' => $billwerkOrderObj->IsTrial ?? NULL,
      '#planName' => $billwerkOrderObj->PlanName ?? NULL,
      '#planVariantName' => $billwerkOrderObj->PlanVariantName ?? NULL,
      '#orderType' => $billwerkOrderObj->OrderType ?? NULL,
      '#contractCustomFields' => [],
      '#customer' => [
        '#theme' => 'billwerk_customer',
        '#companyName' => $billwerkOrderObj->Customer->CompanyName ?? NULL,
        '#firstName' => $billwerkOrderObj->Customer->FirstName ?? NULL,
        '#lastName' => $billwerkOrderObj->Customer->LastName ?? NULL,
        '#addressLine1' => $billwerkOrderObj->Customer->AddressLine1 ?? NULL,
        '#addressLine2' => $billwerkOrderObj->Customer->AddressLine2 ?? NULL,
        '#street' => $billwerkOrderObj->Customer->Street ?? NULL,
        '#houseNumber' => $billwerkOrderObj->Customer->HouseNumber ?? NULL,
        '#country' => $billwerkOrderObj->Customer->Country ?? NULL,
        '#postalCode' => $billwerkOrderObj->Customer->PostalCode ?? NULL,
        '#city' => $billwerkOrderObj->Customer->City ?? NULL,
        '#state' => $billwerkOrderObj->Customer->State ?? NULL,
        '#formattedStreetAddressLine' => $billwerkOrderObj->Customer->FormattedStreetAddressLine ?? NULL,
        '#formattedCityAddressLine' => $billwerkOrderObj->Customer->FormattedCityAddressLine ?? NULL,
        '#vatId' => $billwerkOrderObj->Customer->VatId ?? NULL,
        '#emailAddress' => $billwerkOrderObj->Customer->EmailAddress ?? NULL,
        '#attributes' => [],
      ],
      '#attributes' => [],
    ];
  }

  protected function buildBuyForm(array $form, FormStateInterface $form_state): array {
    $subscriber = Subscriber::loadByDrupalUid($this->currentUser()->id());

    dsm($form_state->getValues());


    $form['actions']['submit'] = [
			'#type' => 'submit',
			'#value' => $this->t('Buy'),
			'#button_type' => 'primary',
		];

    return $form;
  }

	/**
	 * AJAX Callback for the plan variants details.
	 */
	public function updatePlanDetailsCallback(array &$form, FormStateInterface $form_state) {
		return $form['planvariant_details'];
	}

  public static function submitSelectionForm(array &$form, FormStateInterface $form_state) {
		$form_state->set('step', 'review')->setRebuild(TRUE);
	}

  public static function submitReviewForm(array &$form, FormStateInterface $form_state) {
		$form_state->set('step', 'buy')->setRebuild(TRUE);
	}

	/**
	 * {@inheritdoc}
	 */
	public function submitForm(array &$form, FormStateInterface $form_state) {
		// @todo Execute the required JS action or redirect.
		$planvariantMachineName = $form_state->getValue('planvariant');
		$planVariant = $this->subscriptionPlansManager->getBillwerkPlanVariantByMachineName($planvariantMachineName);

		$componentsMachineNames = array_filter($form_state->getValue('components'));
		/** @var \Drupal\billwerk_subscriptions_entities\Entity\BillwerkComponent[] $components */
		$components = [];
		if (!empty($componentsMachineNames)) {
			foreach ($componentsMachineNames as $componentMachineName) {
				$this->subscriptionPlansManager->getBillwerkComponentByMachineName($componentMachineName);
			}
		}
	}

	/**
	 * Checks access for a specific request.
	 *
	 * @param \Drupal\Core\Session\AccountInterface $account
	 *   Run access checks for this account.
	 *
	 * @return \Drupal\Core\Access\AccessResultInterface
	 *   The access result.
	 */
	public static function access(AccountInterface $account) {
		// @todo: Implement!
		return AccessResult::allowed();
	}

}
